'''
Copyright (c)2022 - Soffos.ai - All rights reserved
Created at: 2023-06-26
Purpose: Easily use File Converter Service
-----------------------------------------------------
'''
from typing import Union
from .service import SoffosAIService, inspect_arguments
from soffosai.common.constants import ServiceString


_NORMALIZE_VALUES = [0, 1]

class FileConverterService(SoffosAIService):
    '''
    The File Converter extracts text from various types of files.
    '''

    def __init__(self,  **kwargs) -> None:
        service = ServiceString.FILE_CONVERTER
        super().__init__(service, **kwargs)
    
    def __call__(self, user:str, file:str, normalize:int=0):
        if normalize not in _NORMALIZE_VALUES:
            raise ValueError(f"{self._service}: normalize can only accept a value of 0 or 1")
        payload = inspect_arguments(self.__call__, user, file, normalize)
        return super().__call__(payload)



from .node import Node
from soffosai.core.services import inspect_arguments, TagGenerationService


class TagGenerationNode(Node):
    '''
    Tag Generation Service configuration for Pipeline Use
    '''
    def __init__(self, name:str, text:str, types:list=["topic"], n:int=10):
        source = inspect_arguments(self.__init__, name, text, types, n)
        service = TagGenerationService
        super().__init__(name, service, source)



from .service_io import ServiceIO
from ..constants import ServiceString


class SummarizaionIO(ServiceIO):
    service = ServiceString.SUMMARIZATION
    required_input_fields = ["sent_length", "text"]
    input_structure = {
        "sent_length": int,
        "text": str
    }
    output_structure = {
        "summary": str
    }



'''
Copyright (c)2022 - Soffos.ai - All rights reserved
Created at: 2023-04-01
Purpose: checks the datatype of a dictionary value recursively
-----------------------------------------------------
'''




from .node import Node
from soffosai.core.services import LanguageDetectionService, inspect_arguments


class LanguageDetectionNode(Node):
    '''
    Language Detection configuration for Pipeline Use
    '''
    def __init__(self, name:str, text:str):
        source = inspect_arguments(self.__init__, name, text)
        service = LanguageDetectionService
        super().__init__(name, service, source)



from .service_io import ServiceIO
from ..constants import ServiceString


class DocumentsIngestIO(ServiceIO):
    service = ServiceString.DOCUMENTS_INGEST
    required_input_fields = ["document_name", ]
    require_one_of_choice = [["text", "tagged_elements"]]
    defaults = ["text"]
    optional_input_fields = ["meta"]
    input_structure = {
        "document_name": str,
        "meta": dict,
        "text": str,
        "tagged_elements": [dict, dict]
    }
    # output_fields = ["success", "document_id"]
    output_structure = {
        "success": bool,
        "document_id": str
    }
    primary_output_field = "document_id"


class DocumentSearchIO(ServiceIO):
    service = ServiceString.DOCUMENTS_SEARCH
    required_input_fields = []
    require_one_of_choice = []
    defaults = ["query"]
    optional_input_fields = [
        "query", "filters", "document_ids", "top_n_keywords", 
        "top_n_natural_language", "date_from", "date_until"
    ]
    input_structure = {
        "query": str,
        "document_ids": list,
        "top_n_keyword": int,
        "top_n_natural_language": int,
        "filters": dict,
        "date_from": str,
        "date_until": str
    }
    # output_fields = ["passages"]
    output_structure = {
        "passages": [{
            "content": str,
            "document_id": str,
            "created_at": str,
            "name": str,
            "scores": [
                {
                    "keyword": float,
                    "semantic": float
                },
            ],
            "meta": {}
        },],
        
        "text": str
    }

    @classmethod
    def special_validation(self, payload:dict):
        payload_keys = payload.keys()
        # if 'query' not in payload_keys and 'filters' not in payload_keys:
        #     return False, "If query is not provided, please provide 'filters' argument."
        
        if 'top_n_natural_language' in payload_keys:
            if payload['top_n_natural_language'] > 0 and 'query' not in payload_keys and 'document_ids' not in payload_keys:
                return False, "If document_ids are not defined: query is required if top_n_natural_language is defined and is greater than 0."

        return super().special_validation(payload)


class DocumentDeleteIO(ServiceIO):
    service = ServiceString.DOCUMENTS_DELETE
    required_input_fields = ["document_ids"]
    input_structure = {
        "document_ids": [str, str]
    }
    output_structure = {
        "success": bool
    }



from .node import Node
from soffosai.core.services import LetsDiscussService, inspect_arguments
from soffosai.core.services import LetsDiscussCreateService, LetsDiscussRetrieveService, LetsDiscussDeleteService


class LetsDiscussNode(Node):
    '''
    Lets Discuss Service configuration for Pipeline Use
    '''
    def __init__(self, name:str, session_id:str, query:str):
        source = inspect_arguments(self.__init__, name, session_id, query)
        service = LetsDiscussService
        super().__init__(name, service, source)


class LetsDiscussCreateNode(Node):
    '''
    Lets Discuss Create Service configuration for Pipeline Use
    '''
    def __init__(self, name:str, context:str):
        source = inspect_arguments(self.__init__, name, context)
        service = LetsDiscussCreateService
        super().__init__(name, service, source)


class LetsDiscussRetrieveNode(Node):
    '''
    Lets Discuss Retrieve Sessions Service configuration for Pipeline Use
    '''
    def __init__(self, name:str, return_messages:bool):
        source = inspect_arguments(self.__init__, name, return_messages)
        service = LetsDiscussRetrieveService
        super().__init__(name, service, source)


class LetsDiscussDeleteNode(Node):
    '''
    Lets Discuss Delete Sessions Service configuration for Pipeline Use
    '''
    def __init__(self, name:str, return_messages:bool):
        source = inspect_arguments(self.__init__, name, return_messages)
        service = LetsDiscussDeleteService
        super().__init__(name, service, source)



'''
Copyright (c)2022 - Soffos.ai - All rights reserved
Created at: 2023-06-26
Purpose: Easily use Microlesson Service
-----------------------------------------------------
'''
from .service import SoffosAIService, inspect_arguments
from soffosai.common.constants import ServiceString


class MicrolessonService(SoffosAIService):
    '''
    Accepts a list of texts, each one labelled with its source and creates a concise microlesson 
    including a short summary, key points, learning objectives and tasks that aim to help the 
    learner achieve the learning objectives.
    '''

    def __init__(self,  **kwargs) -> None:
        service = ServiceString.MICROLESSON
        self.content = []
        super().__init__(service, **kwargs)
    

    def __call__(self, user:str, content:list=None):
        if content:
            self.content = content
        payload = inspect_arguments(self.__call__, user, content)
        payload['content'] = self.content
        return super().__call__(payload)


    def add_content(self, source:str, text:str):
        '''
        Add content to the microlesson
        '''
        self.content.append(
            {
                "source": source,
                "text": text
            }
        )



from soffosai.core.services import FileConverterService, SummarizationService, inspect_arguments
from .node import Node


class FileConverterNode(Node):
    '''
    File Converter Node for Pipeline Use
    '''
    def __init__(self, name:str, file) -> None:
        service = FileConverterService
        source = inspect_arguments(self.__init__, name, file)
        super().__init__(name, service, source)



from .service_io import ServiceIO
from ..constants import ServiceString


class ParaphraseIO(ServiceIO):
    service = ServiceString.PARAPHRASE
    required_input_fields = ["text"]
    input_structure = {
        "text": str
    }
    output_structure = {
        "paraphrase": str
    }



'''
Copyright (c)2022 - Soffos.ai - All rights reserved
Created at: 2023-06-30
Purpose: Define the standard Pipeline for converting and summarizing a file
-----------------------------------------------------
'''
from soffosai.core import Node, inspect_arguments
from soffosai.core.nodes import FileConverterNode, SummarizationNode
from soffosai.core.pipelines import Pipeline

class FileSummaryPipeline(Pipeline):
    '''
    A Soffos Pipeline that takes a file, convert it to its text content then summarizes it.
    The output is a list containing the output object of file converter and summarization.
    '''
    def __init__(self, **kwargs) -> None:

        file_converter_node = FileConverterNode(
            name = "fileconverter",
            file = {"source":"user_input", "field": "file"}
        )
        summarization_node = SummarizationNode(
            name = "summary",
            text = {"source":"fileconverter", "field": "text"},
            sent_length = {"source":"user_input", "field": "sent_length"}
        )

        nodes = [file_converter_node, summarization_node]
        use_defaults = False
        super().__init__(nodes=nodes, use_defaults=use_defaults, **kwargs)


    def __call__(self, user, file, sent_length):
        user_input = inspect_arguments(self.__call__, user, file, sent_length)
        return super().__call__(user_input)



'''
Copyright (c)2022 - Soffos.ai - All rights reserved
Created at: 2023-04-21
Purpose: get all service IO definitions
-----------------------------------------------------
'''
import sys
import inspect
from .serviceio_fields import *

attrs = dir()
classes = [getattr(sys.modules[__name__], name) for name in attrs if inspect.isclass(getattr(sys.modules[__name__], name))]

SERVICE_IO_MAP = {obj.service: obj for obj in classes}



from .service_io import ServiceIO
from ..constants import ServiceString

class QuestionAndAnswerGenerationIO(ServiceIO):
    service = ServiceString.QUESTION_AND_ANSWER_GENERATION
    required_input_fields = ["text"]
    optional_input_fields = ["sentence_split", "sentence_overlap"]
    input_structure = {
        "text": str,
        "sentence_split": int,
        "sentence_overlap": bool
    }
    output_structure = {
        "qna_list": [
            {
                "question": str,
                "answer": str,
                "chunk_index": int
            },
        ],

        "chunks": [
            {
                "text": str,
                "span_start": int,
                "span_end": int,
                "index": int
            },
        ]
    }



from .node import Node
from soffosai.core.services import inspect_arguments, TableGeneratorService


TABLE_FORMATS = ['markdonw', 'CSV']

class TableGeneratorNode(Node):
    '''
    Table Generator Service configuration for Pipeline Use
    '''
    def __init__(self, name:str, text:str, table_format:str='markdown'):
        source = inspect_arguments(self.__init__, name, text, table_format)
        service = TableGeneratorService
        super().__init__(name, service, source)



from .service_io import ServiceIO
from ..constants import ServiceString


class LogicalErrorDetectionIO(ServiceIO):
    service = ServiceString.LOGICAL_ERROR_DETECTION
    required_input_fields = ["text"]
    input_structure = {
        "text": str
    }
    output_structure = {
        "logical_errors": [
            {
            "text": str,
            "start": int,
            "end": int,
            "explanation": str
            },
            {
            "text": str,
            "start": int,
            "end": int,
            "explanation": str
            }
        ]
    }



from .service_io import ServiceIO
from ..constants import ServiceString


class ContradictionDetectionIO(ServiceIO):
    service = ServiceString.CONTRADICTION_DETECTION
    required_input_fields = ["text"]
    optional_input_fields = []
    input_structure = {
         "text": str
    }
    output_fields = ["contradictions"]
    output_structure = {
        "contradictions": [
            {
                "contradiction": str,
                "sentences": [
                    {
                        "text": str,
                        "span_start": int,
                        "span_end": int
                    },
                ]
            },
        ]
    }



'''
Copyright (c)2022 - Soffos.ai - All rights reserved
Created at: 2023-06-27
Purpose: Easily use Transcript Correction Service
-----------------------------------------------------
'''
from .service import SoffosAIService, inspect_arguments
from soffosai.common.constants import ServiceString


class TranscriptCorrectionService(SoffosAIService):
    '''
    This module cleans up and corrects poorly transcribed text from Speech-To-Text (STT) systems. 
    It can handle cases where STT produced the wrong word or phrase by taking into account the 
    surrounding context and choosing the most fitting replacement. Although this is meant for correcting 
    STT outpus, it can also be used to correct grammar, misspellings and syntactical errors.
    '''

    def __init__(self,  **kwargs) -> None:
        service = ServiceString.TRANSCRIPTION_CORRECTION
        super().__init__(service, **kwargs)
    

    def __call__(self, user:str, text:str):
        payload = inspect_arguments(self.__call__, user, text)
        return super().__call__(payload)



'''
Copyright (c)2022 - Soffos.ai - All rights reserved
Created at: 2023-06-27
Purpose: Easily use Question and Answer Generation Service
-----------------------------------------------------
'''
from .service import SoffosAIService, inspect_arguments
from soffosai.common.constants import ServiceString


class QuestionAndAnswerGenerationService(SoffosAIService):
    '''
    The Q&A Generation module splits large documents in chunks from which it generates multiple 
    question-answer pairs. The chunk length is configurable. Usually more questions can be generated 
    when segmenting the text to smaller chunks, while longer chunks help retain more context, in cases 
    where a topic is discussed over multiple sentences in the context. To address cases where the topic 
    is split mid-way, the module supports overlapping the chunks by a configurable amount of sentences. 
    This gives a lot of flexibility to cater to your specific use case.
    '''

    def __init__(self,  **kwargs) -> None:
        service = ServiceString.QUESTION_AND_ANSWER_GENERATION
        super().__init__(service, **kwargs)
    

    def __call__(self, user:str, text:str, sentence_split:int=3, sentence_overlap:bool=False):
        payload = inspect_arguments(self.__call__, user, text, sentence_split, sentence_overlap)
        return super().__call__(payload)



from .service_io import ServiceIO
from ..constants import ServiceString


class ProfanityIO(ServiceIO):
    service = ServiceString.PROFANITY
    required_input_fields = ["text"]
    input_structure = {
        "text": str
    }
    output_structure = {
        "profanities": [
            {
                "text": str,
                "span_start": int,
                "span_end": int
            }
        ],
        "offensive_probability": float,
        "offensive_prediction": bool
    }



'''
Copyright (c)2022 - Soffos.ai - All rights reserved
Created at: 2023-06-27
Purpose: Easily use Question Answering Service
-----------------------------------------------------
'''
from .service import SoffosAIService, inspect_arguments
from soffosai.common.constants import ServiceString


class QuestionAnsweringService(SoffosAIService):
    '''
    This module is a combination of various sub-modules that enable users to get accurate answers on 
    questions posed on a large amount of content. It includes basic intent recognition capabilities 
    to enable appropriate responses to incorrect or profane language, or typical personal questions 
    like "How are you?" and greetings
    '''

    def __init__(self,  **kwargs) -> None:
        service = ServiceString.QUESTION_ANSWERING
        super().__init__(service, **kwargs)
    

    def __call__(self, user:str, question:str, document_text:str=None, document_ids:list=None, 
        check_ambiguity:bool=True, check_query_type:bool=True, generic_response:bool=False, meta:dict=None):
        payload = inspect_arguments(self.__call__, user, question, document_text, document_ids, 
        check_ambiguity, check_query_type, generic_response, meta)
        payload['message'] = question
        return super().__call__(payload)




'''
Soffos Inc. Python SDK package
'''

from .client import SoffosAiResponse
from .common.constants import ServiceString
from .core.services import (
    AmbiguityDetectionService, 
    AnswerScoringService, 
    ContradictionDetectionService,
    DocumentsIngestService, DocumentsSearchService, DocumentsDeleteService, DocumentsService,
    EmailAnalysisService,
    EmotionDetectionService,
    FileConverterService,
    LanguageDetectionService,
    LetsDiscussService,
    LogicalErrorDetectionService,
    MicrolessonService,
    NamedEntityRecognitionService,
    ParaphraseService,
    ProfanityService,
    QuestionAndAnswerGenerationService,
    QuestionAnsweringService,
    ReviewTaggerService,
    SentimentAnalysisService,
    SimplifyService,
    SummarizationService,
    TableGeneratorService,
    TagGenerationService,
    TranscriptCorrectionService,
)
from .common.constants import ServiceString

import os

api_key = os.environ.get("SOFFOSAI_API_KEY")

__all__ = [
    "api_key",
    "ServiceString",
    "SoffosAiResponse",
    "AmbiguityDetectionService",
    "AnswerScoringService",
    "ContradictionDetectionService",
    "DocumentsIngestService", "DocumentsSearchService", "DocumentsDeleteService", "DocumentsService",
    "EmailAnalysisService",
    "EmotionDetectionService",
    "FileConverterService",
    "LanguageDetectionService",
    "LetsDiscussService",
    "LogicalErrorDetectionService",
    "MicrolessonService",
    "NamedEntityRecognitionService",
    "ParaphraseService",
    "ProfanityService",
    "QuestionAndAnswerGenerationService",
    "QuestionAnsweringService",
    "ReviewTaggerService",
    "SentimentAnalysisService",
    "SimplifyService",
    "SummarizationService",
    "TableGeneratorService",
    "TagGenerationService",
    "TranscriptCorrectionService"
]

__title__ = "soffosai"
__description__ = "Python SDK for Soffos.ai API"
__version__ = "0.0.3"



from .node import Node
from soffosai.core.services import inspect_arguments, SummarizationService


class SummarizationNode(Node):
    '''
    Summarization Service configuration for Pipeline Use
    '''
    def __init__(self, name:str, text:str, sent_length:int):
        source = inspect_arguments(self.__init__, name, text, sent_length)
        service = SummarizationService
        super().__init__(name, service, source)



from .service_io import ServiceIO
from ..constants import ServiceString


class LetsDiscussCreateIO(ServiceIO):
    service = ServiceString.LETS_DISCUSS_CREATE
    required_input_fields = ["context"]
    input_structure = {
        "context": str
    }
    output_structure = {
        "session_id": str
    }


class LetsDiscussIO(ServiceIO):
    service = ServiceString.LETS_DISCUSS
    required_input_fields = ["session_id", "query"]
    input_structure = {
        "session_id": str,
        "query": str
    }
    output_structure = {
        "response": str,
        "context": str,
        "messages": [
            {
            "text": str,
            "source": str # "user" or "soffos"
            },
        ]
    }


class LetsDiscussRetrieveIO(ServiceIO):
    service = ServiceString.LETS_DISCUSS_RETRIEVE
    required_input_fields = ["return_messages"]
    input_structure = {
        "return_messages": bool
    }
    output_structure = {
        "sessions": [
            {
            "context": str,
            "session_id": str,
            "messages": [
                {
                "query": str,
                "response": str,
                "message_id": int
                },
                {
                "query": str,
                "response": str,
                "message_id": int
                },
            ]
            }
        ],

        "session_count": int
    }


class LetsDiscussDeleteIO(ServiceIO):
    service = ServiceString.LETS_DISCUSS_DELETE
    required_input_fields = ["session_ids"]
    input_structure = {
        "session_ids": list
    }
    output_structure = {
        "success": bool
    }



'''
Copyright (c)2022 - Soffos.ai - All rights reserved
Created at: 2023-06-27
Purpose: Easily use Sentiment Analysis Service
-----------------------------------------------------
'''
from .service import SoffosAIService, inspect_arguments
from soffosai.common.constants import ServiceString


class SentimentAnalysisService(SoffosAIService):
    '''
    This module processes the text to measure whether it is negative, positive or neutral. 
    The text is processed in segments of user-defined length and it provides scores for each 
    segment as well as the overall score of the whole text.
    '''

    def __init__(self,  **kwargs) -> None:
        service = ServiceString.SENTIMENT_ANALYSIS
        super().__init__(service, **kwargs)
    

    def __call__(self, user:str, text:str, sentence_split:int=4, sentence_overlap:bool=False):
        payload = inspect_arguments(self.__call__, user, text, sentence_split, sentence_overlap)
        return super().__call__(payload)



'''
Copyright (c)2022 - Soffos.ai - All rights reserved
Created at: 2023-06-27
Purpose: Easily use Summarization Service
-----------------------------------------------------
'''
from .service import SoffosAIService, inspect_arguments
from soffosai.common.constants import ServiceString


class SummarizationService(SoffosAIService):
    '''
    The summarization module utilizes Natural Language Generation (NLG) to generate an 
    abstractive summary of a specified length. In contrast to extractive summarization methods, 
    which simply calculate the centrality of sentences or passages in the original text and 
    concatenate the highest rated ones, abstractive summaries are often more concise and accurate. 
    The end result isn't necessarily a sum of word-for-word copies of passages from the original text, 
    but a combination of all key points formulated as a new text.
    '''

    def __init__(self,  **kwargs) -> None:
        service = ServiceString.SUMMARIZATION
        super().__init__(service, **kwargs)
    

    def __call__(self, user:str, text:str, sent_length:int):
        payload = inspect_arguments(self.__call__, user, text, sent_length)
        return super().__call__(payload)



'''
Copyright (c)2022 - Soffos.ai - All rights reserved
Created at: 2023-04-19
Purpose: Organize the constants
-----------------------------------------------------
'''
SOFFOS_SERVICE_URL = "https://dev-api.soffos.ai/service/"
# SOFFOS_SERVICE_URL = "http://localhost:8000/service/"

SERVICES_LIST = [
    'ambiguity-detection',
    'answer-scoring', 
    'contradiction-detection', 
    'discuss/create',
    'discuss', 
    'discuss/count',
    'discuss/delete',
    'documents/ingest', 
    'documents/delete',
    'documents/search',
    'email-analysis', 
    'emotion-detection', 
    'file-converter', 
    'flashcard-generation', 
    'intent-classification/confirmation', 
    'intent-classification', 
    'language-detection', 
    'logical-error-detection',
    'microlesson', 
    'ner', 
    'paraphrase', 
    'profanity',
    'qna-generation',
    'question-answering',
    'review-tagger',
    'search-recommendations',
    'sentiment-analysis',
    'simplify',
    'string-similarity',
    'summarization',
    'table-generator',
    'tag',
    'transcript-correction',
    'batch-service/'
]

class ServiceString:
    '''
    Contains the list of Soffos services as attributes
    '''
    AMBIGUITY_DETECTION = 'ambiguity-detection'
    ANSWER_SCORING = 'answer-scoring'
    CONTRADICTION_DETECTION = 'contradiction-detection'
    LETS_DISCUSS_CREATE = 'discuss/create'
    LETS_DISCUSS = 'discuss'
    LETS_DISCUSS_RETRIEVE = 'discuss/count'
    LETS_DISCUSS_DELETE = 'discuss/delete'
    DOCUMENTS_INGEST = 'documents/ingest'
    DOCUMENTS_DELETE = 'documents/delete'
    DOCUMENTS_SEARCH = 'documents/search'
    EMAIL_ANALYSIS = 'email-analysis'
    EMOTION_DETECTION = 'emotion-detection'
    FILE_CONVERTER = 'file-converter'
    FLASHCARD_GENERATION = 'flashcard-generation'
    INTENT_CLASSIFICATION = 'intent-classification'
    LANGUAGE_DETECTION = 'language-detection'
    LOGICAL_ERROR_DETECTION = 'logical-error-detection'
    MICROLESSON = 'microlesson'
    NER = 'ner'
    PARAPHRASE = 'paraphrase'
    PROFANITY = 'profanity'
    QUESTION_AND_ANSWER_GENERATION = 'qna-generation'
    QUESTION_ANSWERING = "question-answering"
    REVIEW_TAGGER = 'review-tagger'
    SEARCH_RECOMMENDATION = 'search-recommendations'
    SENTIMENT_ANALYSIS = 'sentiment-analysis'
    SIMPLIFY = 'simplify'
    STRING_SIMILARITY = 'string-similarity'
    SUMMARIZATION = 'summarization'
    TABLE_GENERATOR = 'table-generator'
    TAG_GENERATION = 'tag'
    TRANSCRIPTION_CORRECTION = 'transcript-correction'
    BATCH_SERVICE = 'batch-service2'


FORM_DATA_REQUIRED = [ServiceString.FILE_CONVERTER,]



from .service_io import ServiceIO
from ..constants import ServiceString


class AnswerScoringIO(ServiceIO):
    service = ServiceString.ANSWER_SCORING
    required_input_fields = ["context", "question", "user_answer"]
    optional_input_fields = ["answer"]
    input_structure = {
        "context": str,
        "question": str,
        "answer": str,
        "user_answer": str
    }
    # output_fields = ["score", "reasoning"]
    output_structure = {
        "score": float,
        "reasoning": str
    }
 


from .service_io import ServiceIO
from ..constants import ServiceString


class SentimentAnalysisIO(ServiceIO):
    service = ServiceString.SENTIMENT_ANALYSIS
    required_input_fields = ["text"]
    optional_input_fields = ["sentence_split", "sentence_overlap"]
    input_structure = {
        "text": str, 
        "sentence_split": int,
        "sentence_overlap": bool
    }
    output_structure = {
        "sentiment_breakdown": [
            {
                "text": str,
                "start": int,
                "end": int,
                "sentiment": {
                    "negative": float,
                    "neutral": float,
                    "positive": float
                }
            },
        ],
        "sentiment_overall": {
            "negative": float,
            "neutral": float,
            "positive": float
        }
    }



from .service_io import ServiceIO
from ..constants import ServiceString


class AmbiguityDetectionIO(ServiceIO):
    service = ServiceString.AMBIGUITY_DETECTION
    required_input_fields = ["text"]
    optional_input_fields = ["sentence_split", "sentence_overlap"]
    input_structure = {
        "text": str,
        "sentence_split": int,
        "sentence_overlap": bool
    }
    # output_fields = ["ambiguities"]
    output_structure = {
        "ambiguities": {
            "text": str,
            "span_start": int,
            "spane_end": int,
            "reason": str        
        }
    }
